const { alert, okError } = require("./lib/dialogs.js");
const { exportJson, imageExport, getImageExportArray, getSubFolder, resetPlugin, getUniqueArtboardsFromSelection, getUniqueArtboardsFromAll, convertColorStyleList } = require('./utils')
const { isAnyArtboardSelected, checkNonExportableArtboards, checkInternetConnection } = require('./validators')
const fs = require("uxp").storage.localFileSystem
const assets = require("assets")

const portRange = [9221, 9250];
const RESPONSE_TIME = 120; // in ms
const PORT_CHECKING_KEY = 'handshake';
const PORT_REQUEST_KEY = 'monday';
const PORT_RESPONSE_KEY = 'hero';

async function clearFolder(entry) {
  if (entry.isFolder) {
    var folderEntries = await entry.getEntries()
    for (var i = 0; i < folderEntries.length; i++) {
      await clearFolder(folderEntries[i])
    }
  }
  await entry.delete()
}

async function clearPluginDataFolder() {
  var folder = await fs.getDataFolder()
  const entries = await folder.getEntries();
  for (var i = 0; i < entries.length; i++) {
    try {
      await clearFolder(entries[i])
    } catch (e) {
      console.log("Error: " + e)
    }
  }
}

async function exportSelectedArtboards(selection, documentRoot) {
  if (!await isAnyArtboardSelected(selection)) {
    return alert("Export to Monday Hero",
      "No artboard(s) selected.")
  }
  var uniqueArtboards = await getUniqueArtboardsFromSelection(selection)
  return await openWebSocketConnection(uniqueArtboards, documentRoot)
}

async function exportAllArtboards(selection, documentRoot) {
  var uniqueArtboards = await getUniqueArtboardsFromAll(documentRoot)
  return await openWebSocketConnection(uniqueArtboards, documentRoot)
}

function connect(port) {
  return new Promise(function (resolve, reject) {
      let server = new WebSocket(`ws://localhost:${port}`);
      let isConnected = false;
      let isRequestSent = false;

      setTimeout(() => {
          if (!isRequestSent) {
            reject('Port is empty');
          }
      }, RESPONSE_TIME);

      server.onmessage = async (event) => {
          console.log(`Data received from server: ${event.data}`);
          if (event.data === PORT_RESPONSE_KEY) {
            isConnected = true;
            resolve(server);
          }
          isConnected = false;
          reject('Port does not belong to Monday Hero');
      };

      server.onopen = async event => {
          var request = {
            "actionCode": PORT_CHECKING_KEY,
            "payload": PORT_REQUEST_KEY
          };
          console.log(`Trying url ${server.url}`);
          isRequestSent = true;
          server.send(JSON.stringify(request));
          setTimeout(() => {
              if (!isConnected) {
                reject('Port does not belong to Monday Hero');
              }
          }, RESPONSE_TIME);
      };
      server.onerror = async event => {
          isConnected = false;
          reject(event);
      };
  });
}
 
async function getWebSocket() {
  for (let i = portRange[0]; i <= portRange[1]; i++) {
      try {
          const server = await connect(i);
          return server;
      } catch (e) {}
  }
  return null;
}

async function openWebSocketConnection(uniqueArtboards, documentRoot) {
  try {
    if (! await checkInternetConnection()) {
      return alert("Connect to the Internet",
        "In order to function correctly, this plugin requires Internet access. Please connect to a network that has Internet access."
      );
    }
    let ws = await getWebSocket();
    if (!ws) {
      console.log("Unavailable Port, Monday Hero App is not reachable");
      return await alert("Open Monday Hero App",
        "To export your artboards, Monday Hero needs to be open.",
        " ",
        "## MORE INFORMATION",
        "-----",
        "[If you don't already have the Monday Hero App, please download it here.](https://mondayhero.io/download)"
      );
    }
    
    console.log("Plugin connected to Monday Hero");

    ws.onclose = async event => {
      console.log("Plugin disconnected to Monday Hero")
    };
    
    ws.onerror = async event => {
      console.log("Plugin couldn't connect to Monday Hero")
      return await alert("Open Monday Hero App",
        "To export your artboards, Monday Hero needs to be open.",
        " ",
        "## MORE INFORMATION",
        "-----",
        "[If you don't already have the Monday Hero App, please download it here.](https://mondayhero.io/download)"
      )
    }
    if (await prepareDesigns(uniqueArtboards, documentRoot) === "Success") {
      const dataFolder = await fs.getDataFolder()
      const nativeDataPath = await fs.getNativePath(dataFolder)
      var startExportRequest = {
        "actionCode": "start",
        "source": "adobeXd",
        "fileName": "adobeXdDesign.xdmhero",
        "dataPath": nativeDataPath
      }
      ws.send(JSON.stringify(startExportRequest))
    }
    ws.close();
  } catch (e) {
    console.log("Error: " + e)
  }
}

async function prepareDesigns(uniqueArtboards, documentRoot) {
  await clearPluginDataFolder()
  if (await checkNonExportableArtboards(uniqueArtboards)) return
  await resetPlugin()
  var exportJsonData = []
  for (var i = 0; i < uniqueArtboards.length; i++) {
    var exportJsonArray = await exportJson(uniqueArtboards[i], uniqueArtboards[i]);
    var artboardGuid = exportJsonArray.sceneNode.guid
    exportJsonData.push({
      "json": exportJsonArray,
      "artboardId": artboardGuid
    })
  }
  var folder = await fs.getDataFolder()
  folder = await folder.createFolder("symbol_masters");
  const symbolMastersFile = await folder.createFile("symbol_masters.json", { overwrite: true })
  
  //SymbolMasters canceled.
  var symbolMasters = [];
  symbolMastersFile.write(JSON.stringify(symbolMasters));

  // Styles
  var folder = await fs.getDataFolder()  
  folder = await folder.createFolder("styles");
  const textStylesFile = await folder.createFile("text_styles.json", { overwrite: true })
  const colorStylesFile = await folder.createFile("colors.json", { overwrite: true })

  const textStyles = assets.characterStyles.get();
  textStylesFile.write(JSON.stringify(textStyles));
  const colors = assets.colors.get();
  const convertedColors = convertColorStyleList(colors)
  colorStylesFile.write(JSON.stringify(convertedColors));

  for (var i = 0; i < exportJsonData.length; i++) {
    var artboardGuid = exportJsonData[i].json.sceneNode.guid
    var folder = await fs.getDataFolder()
    folder = await getSubFolder(folder, artboardGuid)
    const newFile = await folder.createFile(artboardGuid + ".json", { overwrite: true })
    newFile.write(JSON.stringify(exportJsonData[i].json))
  }
  var imageExportArray = await getImageExportArray()
  if (imageExportArray.length > 0) await imageExport(imageExportArray, uniqueArtboards)
  return "Success"
}

module.exports = {
  commands: {
    exportSelectedArtboards,
    exportAllArtboards
  }
};
