const fs = require("uxp").storage.localFileSystem;
const application = require("application");
const { Text, Rectangle, Group, Ellipse, Polygon, Line, Path, Artboard, BooleanGroup, SymbolInstance, RepeatGrid, ScrollableGroup, RootNode, Color } = require('scenegraph')
const { RectangleModel, TextModel, EllipseModel, PolygonModel, LineModel, PathModel, GroupModel, BooleanGroupModel, SymbolInstanceModel,
    RepeatGridModel, ScrollableGroupModel, ArtboardModel } = require('./AdobeLayerModels')

var imageExportArray = []

async function getSubFolder(folder, subfolderName, create = true) {
    var subfolder = null;
    try {
        subfolder = (await folder.getEntries()).find(entry => entry.name.includes(subfolderName));
        if (subfolder == null || subfolder.isFolder == false) {
            if (create) {
                subfolder = await folder.createFolder(subfolderName);
            }
        }
    }
    catch (fileError) {
        console.log("File Error")
        return fileError;
    }
    return subfolder;
}

function isExportable(node) {
    return node.markedForExport || node instanceof Artboard || node instanceof BooleanGroup // || isExportableGroup(node) 
        || isNotCircleEllipse(node) || node instanceof Path || node instanceof Line
}

function isNotCircleEllipse(node) {
    return (node instanceof Ellipse && node.isCircle !== true);
}

function isExportableGroup(node) {
    var check = false
    if (node instanceof Group || node instanceof BooleanGroup) {
        check = true
        node.children.forEach(function (childNode, i) {
            if (!(childNode instanceof Path)) {
                check = false
            }
        });
    }
    return check
}

function updateBoundsInParentCoordinate(node, artboardNode) {
    var newBoundsParent = {}
    newBoundsParent.width = node.sceneNode.globalDrawBounds.width
    newBoundsParent.height = node.sceneNode.globalDrawBounds.height
    newBoundsParent.x = node.sceneNode.globalDrawBounds.x - artboardNode.globalDrawBounds.x
    newBoundsParent.y = node.sceneNode.globalDrawBounds.y - artboardNode.globalDrawBounds.y
    return newBoundsParent
}

var symbolMasters = []
var symbolInstances = []
var searchSymbolMasters = []
function exportJson(node, artboardNode) {
    var instanceNode;
    if (node instanceof Rectangle) {
        instanceNode = new RectangleModel(node);
    } else if (node instanceof Text) {
        instanceNode = new TextModel(node);
    } else if (node instanceof Ellipse) {
        instanceNode = new EllipseModel(node);
    } else if (node instanceof Polygon) {
        instanceNode = new PolygonModel(node);
    } else if (node instanceof Line) {
        instanceNode = new LineModel(node);
    } else if (node instanceof Path) {
        instanceNode = new PathModel(node);
    } else if (node instanceof Group) {
        instanceNode = new GroupModel(node);
    } else if (node instanceof BooleanGroup) {
        instanceNode = new BooleanGroupModel(node);
    } else if (node instanceof SymbolInstance) {
        instanceNode = new SymbolInstanceModel(node);
    } else if (node instanceof RepeatGrid) {
        instanceNode = new RepeatGridModel(node);
    } else if (node instanceof ScrollableGroup) {
        instanceNode = new ScrollableGroupModel(node);
    } else if (node instanceof Artboard) {
        instanceNode = new ArtboardModel(node);
    }
    
    instanceNode.sceneNode.boundsInParent = updateBoundsInParentCoordinate(instanceNode, artboardNode);

    if (!instanceNode.sceneNode.visible) {
        return instanceNode
    }

    if (instanceNode.mask) {
        instanceNode.sceneNode.markedForExport = true;
        imageExportArray.push(node);
        return instanceNode;
    }

    if (isExportable(node)) {
        imageExportArray.push(node);
        instanceNode.sceneNode.markedForExport = true
        if (!(node instanceof Artboard || node instanceof SymbolInstance)) {
            return instanceNode
        }
    }

    if (node.children.length > 0) {
        var childrenArray = []
        node.children.forEach(function (childNode, i) {
            var element = exportJson(childNode, artboardNode);
            childrenArray.push(element)
        });
        if ((node instanceof SymbolInstance) && (!node.isMaster) && instanceNode.sceneNode.visible) {
            instanceNode.overrideChildren = childrenArray
            symbolInstances.push(instanceNode)
        } else if ((node instanceof SymbolInstance) && (node.isMaster)) {
            instanceNode.children = childrenArray
            symbolMasters.push(instanceNode)
        } else {
            instanceNode.children = childrenArray
        }
        return instanceNode
    } else {
        return instanceNode
    }
}

function createRenditionOption(node, outputFile, type, scale, minify, embedImages) {
    return {
        node,
        outputFile,
        type,
        scale,
        minify,
        embedImages
    }
}

function isSvg(type) {
    return type === application.RenditionType.SVG;
}

function shouldMinify(type) {
    return isSvg(type);
}

function shouldEmbedImages(type) {
    return isSvg(type);
}

function defaultFileName(guid, type) {
    return guid + "." + type;
}

function scaledFileName(guid, scale, type) {
    return guid + "@" + scale.toString() + "x" + '.' + type;
}

async function createRenditionOptions(node, folder) {
    var renditionOptions = [];
    var scales = [1, 1.5, 2, 3, 4];
    const SINGLE_SCALE = 1;
    var renditionTypes = [application.RenditionType.SVG, application.RenditionType.PDF];
    var pngType = application.RenditionType.PNG;

    for (var scale of scales) {
        if (scale === 1) {
            renditionOptions.push(createRenditionOption(
                node,
                await folder.createFile(defaultFileName(node.guid, pngType), { overwrite: true }),
                pngType,
                scale
            ))
            if (node instanceof Artboard) break
        } else {
            renditionOptions.push(createRenditionOption(
                node,
                await folder.createFile(scaledFileName(node.guid, scale, pngType), { overwrite: true }),
                pngType,
                scale
            ))
        }
    }

    for (var type of renditionTypes) {
        renditionOptions.push(createRenditionOption(
            node,
            await folder.createFile(defaultFileName(node.guid, type), { overwrite: true }),
            type,
            SINGLE_SCALE,
            shouldMinify(type), // necessary flag for the svg format
            shouldEmbedImages(type) // necessary flag for the svg format
        ))
    }
    return renditionOptions;
}

async function isChildNodeOfSymbol(node) {
    while (!(node instanceof SymbolInstance) && node) {
        if (node.parent) {
            node = node.parent
        } else {
            break
        }
    }
    if (node instanceof SymbolInstance) {
        return true
    }
    return false
}

async function getParentSymbol(node) {
    while (!(node instanceof SymbolInstance) && node) {
        if (node.parent) {
            node = node.parent
        } else {
            break
        }
    }
    if (node instanceof SymbolInstance) {
        return node
    }
    return node
}

async function imageExport(imageExportArray, uniqueArtboards) {
    var renditionOptions = []
    for (var i = 0; i < imageExportArray.length; i++) {
        var folder = await fs.getDataFolder();
        var node = imageExportArray[i]
        var artboardNode = getArtboardOfNode(node)
        if (uniqueArtboards.includes(artboardNode)) {
            folder = await getSubFolder(folder, artboardNode.guid)
            renditionOptions = renditionOptions.concat(await createRenditionOptions(imageExportArray[i], folder));
        }
    }
    try {
        const results = await application.createRenditions(renditionOptions);
    } catch (error) {
        console.log(error)
    }
}

async function getImageExportArray() {
    return imageExportArray;
}

async function resetPlugin() {
    imageExportArray = []
    symbolMasters = []
    symbolInstances = []
    searchSymbolMasters = []
}

async function getUniqueArtboardsFromSelection(selection) {
    var uniqueArtboards = []
    for (var i = 0; i < selection.items.length; i++) {
        var node = selection.items[i];
        node = getArtboardOfNode(node)
        if (node instanceof Artboard) {
            if (!uniqueArtboards.includes(node)) {
                uniqueArtboards.push(node)
            }
        }
    }
    return uniqueArtboards
}

async function getUniqueArtboardsFromAll(documentRoot) {
    var uniqueArtboards = []
    documentRoot.children.forEach(node => {
        if (node.constructor.name == "Artboard") {
            uniqueArtboards.push(node);
        }
    });
    return uniqueArtboards
}

function getArtboardOfNode(node) {
    while (!(node instanceof Artboard || node instanceof ArtboardModel || node instanceof RootNode)) {
        if (node.parent) {
            node = node.parent
        } else if (node.sceneNode.parent) {
            node = node.sceneNode.parent
        } else {
            break
        }
    }
    return node
}

function convertColorStyleList(colorStyleList) {
    const convertedColorStyleList = []
    if (colorStyleList.length) {
        colorStyleList.forEach(colorStyle => {
            if (colorStyle.color instanceof Color) {
                let rgba = colorStyle.color.toRgba()
                convertedColorStyleList.push({ ...colorStyle, rgba })
            }
        })
    }
    return convertedColorStyleList
}

module.exports = {
    exportJson,
    imageExport,
    getImageExportArray,
    getSubFolder,
    resetPlugin,
    getUniqueArtboardsFromSelection,
    getUniqueArtboardsFromAll,
    convertColorStyleList
}